--handles item behavior/interactions with the boat physics
local MAPBLOCKUNITS = 128
local MAPBLOCKSIZE = MAPBLOCKUNITS*FU
local function randomitemscale(oldscale)
	local newscale = oldscale * 3
	local maxscale = FixedDiv(MAPBLOCKSIZE, mobjinfo[MT_RANDOMITEM].radius)
	return min(newscale, maxscale)
end

local spawninglist = {}
local thinkerlist = {}
local chargefalllist = {}

local function trapitem_spawn(item)
	if not (mapheaderinfo[gamemap] and mapheaderinfo[gamemap].boat ~= nil) then return end
	
	table.insert(spawninglist, item)
end

addHook("MobjSpawn",trapitem_spawn,MT_BANANA)
addHook("MobjSpawn",trapitem_spawn,MT_EGGMANITEM)
addHook("MobjSpawn",trapitem_spawn,MT_ORBINAUT) --special
--jawz should be fine, it automatically waterruns anyways
----FIXME: later
--addHook("MobjSpawn",trapitem_spawn,MT_SSMINE_SHIELD)
----doesnt set water surfaces
--addHook("MobjSpawn",trapitem_spawn,MT_LANDMINE)
----doesnt set water surfaces
--addHook("MobjSpawn",trapitem_spawn,MT_DROPTARGET)
addHook("MobjSpawn",trapitem_spawn,MT_GACHABOM)

addHook("MobjSpawn",trapitem_spawn,MT_CHARGEFALL) --hacky

addHook("NetVars", function(n)
	thinkerlist = n($)
	spawninglist = n($)
	chargefalllist = n($)
end)

addHook("PreThinkFrame",do
	--handle spawning here, since it needs to be delayed in order
	--to get the item's target
	for k,item in ipairs(spawninglist)
		if not (item and item.valid)
			table.remove(spawninglist, k)
			continue
		end
		
		if not ((item.target and item.target.valid)
		and (item.target.player and item.target.player.valid))
			return
		end
		
		local me = item.target
		local p = me.player
		
		if (p.boatdriving)
		or (me.boat_tilting)
			table.insert(
				(item.type == MT_CHARGEFALL) and chargefalllist or thinkerlist,
				item
			)
		end
		
		table.remove(spawninglist, k)
		continue
	end

	--then we can iterate through the items
	for k,item in ipairs(thinkerlist)
		if not (item and item.valid)
		or not item.health
			table.remove(thinkerlist, k)
			continue
		end
		
		if (item.type == MT_LANDMINE)
		and item.threshold
			continue
		end
		
		local origin = (item.target and item.target.valid) and item.target or item
		local wasinwater = item.inwater
		
		local gravflip = P_MobjFlip(item)
		local sink = (item.height / 5)
		local z = item.z
		local water_top = item.watertop - sink
		do	
			if (gravflip == -1)
				water_top = item.waterbottom + sink
				z = item.z+item.height
			end
			
			/*
			print(string.format("land %f\n%f\n%s\n%s\n%f",
				z, water_top,
				tostring(water_top ~= item.z - 1000*FU),
				tostring(z < water_top),
				item.height
			))
			*/
			if (water_top ~= item.z - 1000*FU)
			and (z < water_top)
				item.inwater = true
				
				if not wasinwater
					if not (item.type == MT_ORBINAUT
					or item.type == MT_GACHABOM)
						item.momx = $ / 5
						item.momy = $ / 5
					end
					item.momz = $ / 5
					
					if item.bobs == nil
						item.bobs = 0
					end
					item.bobs = $ + 1
				end
			end
		end
		
		if item.inwater
			if (abs(water_top - z) < 3*item.scale)
			and abs(item.momz) <= 5*item.scale
				item.momz = $*2/3
				
				if abs(item.momz) <= 3*item.scale
					item.momz = $/3
				end
				
			--buoyant
			else
				item.flags = $|MF_NOGRAVITY
				local floating = (origin.scale*3/2)*gravflip
				local diff = (water_top - z) * gravflip
				if diff < 0
					floating = -$
				end
				
				item.momz = $ + floating --FixedDiv(abs(diff/100), abs(floating))
			end
			item.waterskip = max($,3)
			
			if (item.type == MT_BANANA)
			or (item.type == MT_EGGMANITEM)
			or (item.type == MT_SSMINE)
				item.extravalue2 = 0
				
				if item.bobs
					item.momz = $ / item.bobs
				end
				item.momx = $ / 2
				item.momy = $ / 2
				
				if (item.type == MT_BANANA)
				or (item.type == MT_EGGMANITEM)
					if abs(water_top - z) < 10*item.scale
						item.momz = 0
					end
					if (item.momz*gravflip <= -10*origin.scale)
						item.momz = -10*origin.scale
					end
					
					if item.health > 1
						S_StartSound(item, item.info.activesound)
						item.health = 1
						item.momx,item.momy = 0,0
						
						if (item.type == MT_EGGMANITEM)
							--love it
							item.destscale = randomitemscale(item.scale)
						end
					end
				end
			end
		end
		
		if (wasinwater and not item.inwater)
			--item.flags = $ &~MF_NOGRAVITY
			
			item.momz = 0
			item.momx = $ / 5
			item.momy = $ / 5
		end
		
	end
end)

-- .
addHook("PostThinkFrame", do
	for k,item in ipairs(chargefalllist)
		if not (item and item.valid)
		or not item.health
			table.remove(chargefalllist, k)
			continue
		end
		
		-- Lol!
		item.renderflags = $ &~RF_DONTDRAW
	end
end)

--lul
addHook("MobjThinker",function(stumble)
	local me = stumble.target
	if not (me and me.valid) then return end
	local p = me.player
	if not (p and p.valid) then return end
	
	--we wont stumble in this condition
	if me.boat_tilting
		stumble.renderflags = $|RF_DONTDRAW
	end
end,MT_SMOOTHLANDING)